//----------------------------------------------
// STAR LIGHT -- Double precision version
//----------------------------------------------
// Draws a rotating pyramid, and a rotating
//   light source.
// Demonstrates use of the Object, Scene, and
//   Light classes, as well a fixed point math.
//----------------------------------------------
// Copyright (C) 1993, Nicholas Centanni
//----------------------------------------------
#include <graphics.h>
#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <conio.h>
#include <malloc.h>
#include <math.h>
#include <bios.h>
#include <op_color.h>
#include <op_key.h>
#include <op_scene.h>
#include "d_pyramd.h"
#include "d_star.h"

#define VIEW_WINDOWS   1
#define VIDEO_PAGES    2
#define MAX_OBJECTS    3
#define ORBIT_CYCLE  100

Viewer viewer;
Graphics graphics;
Vect vect;
d_Light light(1, 0);
d_Environ env(&viewer, &graphics, &vect, &light, VIDEO_PAGES, VIEW_WINDOWS);
d_Scene scene(&env, MAX_OBJECTS);
d_Star *star;
d_Pyramid *pyramid, *another;

// Define a light source
d_source light_source =
{
  { 0, 0, 0 },
  { 0.8, 0.8, 0.8 },
};
d_point center = { 0, 0, 0 };
int heading = 0;

//-----------------------------------------------------------------------
// Program MAIN
//-----------------------------------------------------------------------
void main()
{
  int orbit_on = 0;
  int orbit_cnt = ORBIT_CYCLE;

  // Start graphics
  scene.start_graphics(EGA_16);
  graphics.paging_on();
  setallpalette(&earth);

  // Set up viewing area
  viewer.set_zoom(80);

  // Define the initial viewing position
  viewer.set_position(-1400, 150, 1400);
  viewer.set_center(-400, 300, 200);
  viewer.view_center();
  viewer.new_view();

  // Add the first pyramid
  pyramid = new d_Pyramid(&env);
  pyramid->set_center(0, 200, 0);
  scene.add_object(pyramid);

  // Clone the pyramid to a second copy
  another = pyramid->clone(&env);
  another->move(-800, 0, 400);
  another->offset_shades(4);
  scene.add_object(another);

  // Add the light source-star
  star = new d_Star(&env);
  scene.add_object(star);

  // Define lighting for the scene
  star->define_orbit(0, &center);
  star->orbit(heading);
  star->copy_vertex(0, &light_source.world);
  star->set_center(light_source.world.x, light_source.world.y,
                   light_source.world.z);
  scene.set_light_source(0, &light_source);

  // Draw the first frame of the scene on both PAGES
  graphics.chg_active_page();
  scene.translate();
  scene.sort_objects();
  scene.draw(ON);
  graphics.chg_active_page();
  scene.translate();
  scene.sort_objects();
  scene.draw(ON);

  // Run the animation
  while (!kbhit())
  {
    // Erase the previous frame
    graphics.chg_active_page();
    scene.draw(OFF);

    // Orbit around the scene
    if (orbit_on)
    {
      viewer.orbit(12);
      viewer.chg_heading(12);
      viewer.new_view();
    }

    // Process the orbit counter
    if (--orbit_cnt == 0)
    {
      orbit_on = !orbit_on;
		orbit_cnt = ORBIT_CYCLE + random(100);
    }

    // Orbit the STAR and LIGHT_SOURCE around the pyramid
    heading += 36;
    viewer.clip_angle(&heading);
    star->orbit(heading);
    star->copy_vertex(0, &light_source.world);
    star->set_center(light_source.world.x, light_source.world.y,
                     light_source.world.z);
    scene.set_light_source(0, &light_source);

    // Draw the current scene
	 scene.translate();
	 scene.sort_objects();
	 scene.draw(ON);
	 graphics.reset();

	 // Flip pages
	 graphics.chg_visual_page();
	 delay(10);
  }
  bioskey(0);

  // Return to text mode
  scene.end_graphics();
}

