//----------------------------------------------
// H_CUBE -- XYZ++ example program
//----------------------------------------------
// Orbits around a cube drawn with hidden-lines
// removed.
//----------------------------------------------
// Copyright (C) 1993, Nicholas Centanni
//----------------------------------------------
#include <graphics.h>
#include <dos.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
#include <op_graph.h>
#include <op_view.h>

#define VIEW_SCREEN   0
#define UP            1
#define DOWN          2

Graphics graphics;       // Declare a graphics object
Viewer viewer;           // Declare a 3-D viewer object

// Define the cube size
#define SIZE      60
#define SIZE_2   (SIZE * 2)

// Define the cube center coordinates
#define CX        10
#define CY        10
#define CZ        10

// Define a vertex for the center of the cube
l_vertex center = { CX, CY, CZ };

// Define the cube vertex structure
struct cube_point
{
  l_point world;    // World coordinates
  i_screen pix;     // Screen coordinates
};

// Define the cube
struct cube_point cube[] =
{
  { CX - SIZE, CY - SIZE, CZ - SIZE },    // Bottom four vertices
  { CX + SIZE, CY - SIZE, CZ - SIZE },
  { CX + SIZE, CY - SIZE, CZ + SIZE },
  { CX - SIZE, CY - SIZE, CZ + SIZE },

  { CX - SIZE, CY + SIZE, CZ - SIZE },    // Top four vertices
  { CX + SIZE, CY + SIZE, CZ - SIZE },
  { CX + SIZE, CY + SIZE, CZ + SIZE },
  { CX - SIZE, CY + SIZE, CZ + SIZE },
};

// The cube_side array will keep track of the vertices
// of each of the six cube sides
struct cube_side
{
  int corner[4];     // The four corners
  int visible;       // Visibility state of the face
};

// Initialize the cube side descriptions
// The sides are describes as lists of vertices taken
// counter-clockwise when viewed from outside the cube
struct cube_side sides[] =
{
  { 0, 1, 2, 3 },  // Bottom
  { 1, 0, 4, 5 },  // North
  { 1, 5, 6, 2 },  // East
  { 3, 2, 6, 7 },  // South
  { 0, 3, 7, 4 },  // West
  { 7, 6, 5, 4 },  // Top
};

// Supporting functions
void display(int color);
void animate(void);

//-------------------------------------------------------------------------
// Program main
//-------------------------------------------------------------------------
void main()
{
  // Start graphics
  registerbgidriver(EGAVGA_driver);
  graphics.on(VGA, VGAHI, "");
  delay(800);  // Give auto-sync screens a chance to settle down

  // Define an animation-capable viewing window
  graphics.define_port(VIEW_SCREEN, 0, 0, getmaxx(), getmaxy());
  graphics.set_port(VIEW_SCREEN);

  // The 3-D zoom setting MUST take place AFTER
  // the above calls to define_port and set_port
  viewer.set_zoom(100);

  // Run the animation until a key is pressed
  animate();

  // Return to text mode
  graphics.off();
}

//-------------------------------------------------------------------------
// Move past the object in space
//-------------------------------------------------------------------------
void animate()
{
  int obj_color = LIGHTCYAN;
  int rotate_chg = 20;

  // Place the viewer at a position in space: x, y, z
  viewer.set_position(150, 150, 250);

  // Set the center of interest to be the middle of the cube
  viewer.set_center(center.pos.x, center.pos.y, center.pos.z);
  viewer.view_center();  // Aim the viewer at the center

  // Display the object
  display(obj_color);

  // Animate
  while (!kbhit())
  {
    // Erase the object
    display(BLACK);

    // Change the viewer position
    viewer.orbit(rotate_chg);
    viewer.chg_heading(rotate_chg);

    // Redisplay the object from the new angle
    display(obj_color);

    // Move slowly
    delay(40);
  }

  // Read the exit keystroke so it
  // doesn't pass on to DOS
  getch();
}

//-------------------------------------------------------------------------
// Display the cube at its current position,
// from the current point of view.
//-------------------------------------------------------------------------
void display(int color)
{
  int i;
  long v1, v2, w1, w2;

  // Only calculate coordinates when drawing the visible image
  if (color != BLACK)
  {
    // Calculate the viewing matrix for this angle of view
    viewer.new_view();

    // Calculate the 3-D location of each cube vertex
    for (i = 0; i < 8; i++)
      viewer.translate(&cube[i].world, &cube[i].pix);

    // Determine the visibility of each cube side
    for (i = 0; i < 6; i++)
    {
      struct cube_side *s = &sides[i];
      i_screen *px0 = &cube[s->corner[0]].pix;
      i_screen *px1 = &cube[s->corner[1]].pix;
      i_screen *px2 = &cube[s->corner[2]].pix;

      // Determine the orientation of the face
      v1 = px1->x - px0->x;
      v2 = px1->y - px0->y;
      w1 = px2->x - px1->x;
      w2 = px2->y - px1->y;

      // Store the visibility flag
      sides[i].visible = ((v1 * w2 - v2 * w1) < 0);
    }
  }

  // Draw the edge lines of the visible faces
  setcolor(color);
  for (i = 0; i < 6; i++)
  {
    if (sides[i].visible)
    {
      moveto(cube[sides[i].corner[3]].pix.x,
             cube[sides[i].corner[3]].pix.y);

      for (int j = 0; j < 4; j++)
        lineto(cube[sides[i].corner[j]].pix.x,
               cube[sides[i].corner[j]].pix.y);
    }
  }
}

