//------------------------------------------------
// Surface Lighting
//------------------------------------------------
#include <graphics.h>
#include <dos.h>
#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <op_graph.h>
#include <op_key.h>
#include <op_view.h>
#include <op_vect.h>
#include <op_light.h>

Graphics graphics;
Viewer viewer;
Keyboard keyboard;
Vect vect;
d_Light light(1, 60);

// Definitions
#define VIEWPORT      0
#define HELPPORT      1

#define MORE_RED      0
#define MORE_GREEN    1
#define MORE_BLUE     2
#define LESS_RED      3
#define LESS_GREEN    4
#define LESS_BLUE     5
#define LIGHT_UP      6
#define LIGHT_DOWN    7
#define LIGHT_EAST    8
#define LIGHT_WEST    9
#define NO_EVENT     -1
#define EXIT         -2

l_point lt_pos = {800, 400, 800};
d_color lt_color = {1.0, 1.0, 1.0};
i_screen lt_source = {0, 0, 0};
int poly[4];

// Supporting functions
void draw_scene(int state);
void display_surface(void);
void display_light(void);
void define_surface(d_plane *p, d_point *base, d_point *v1, d_point *v2);
long key_read(void);
long get_key(void);
void display_color_values(int which = -1);

d_surface my_surface =
{
  { 0, 3, 2, 1 },      // v_index
  { 1.0, 1.0, 1.0 },   // color
  0.23,                // ambient
  0.87,                // diffuse
  0,                   // shade
  4,                   // edge_tot
};

d_vertex square[4] =
{
  //- x   y   z -//
  {{ 400, 0, 400 }},
  {{ 1200, 0, 400 }},
  {{ 1200, 0, 1200 }},
  {{ 400, 0, 1200 }},
};

//-------------------------------------------------------------------------
// Program MAIN
//-------------------------------------------------------------------------
void main()
{
  // Start graphics
  registerbgidriver(EGAVGA_driver);
  graphics.on(EGA, EGAHI, "");
  delay(800);  // Give auto-sync screens a chance to settle down

  // Define a viewing window - there MUST be at least one
  graphics.define_port(VIEWPORT, 0, 0, getmaxx(), getmaxy()-51, PAGING_ON);
  graphics.define_port(HELPPORT, 0, getmaxy()-50, getmaxx(), getmaxy(), PAGING_ON);
  graphics.set_port(VIEWPORT);
  graphics.paging_on();
  graphics.reset_pages();

  // The 3-D zoom setting MUST take place AFTER
  // the above calls to define_port and set_port
  viewer.set_zoom(70);

  // Define our light sources
  light.define_source(0, lt_pos.x, lt_pos.y, lt_pos.z, 1.0, 1.0, 1.0);

  // Display Help Text
  display_color_values();

  // Calculate the coefficients of our plane
  d_surface *s = &my_surface;
  d_point center = {800, 0, 800};
  define_surface(&s->plane, &center,
                 &square[s->v_index[1]].pos, &square[s->v_index[2]].pos);

  // Calculate the surface color
  light.determine(s);

  // Place the viewer at a position in space: x, y, z
  viewer.set_position(1800, 250, 1800);

  // Set the center of interest to be the middle of the square: x, y, z
  viewer.set_center(800, 0, 800);
  viewer.view_center();  // Look at the center of interest
  viewer.new_view();

  // Draw the initial image         
  graphics.chg_active_page();
  draw_scene(ON);
  graphics.chg_active_page();
  draw_scene(ON);

  int done = 0;
  long key_event;
  while (!done)
  {
    // Erase the previous image
    graphics.chg_active_page();
    draw_scene(OFF);

    // Orbit around the scene
    viewer.orbit(10);
    viewer.chg_heading(10);
    viewer.new_view();

    double inc = 20;
    double c_inc = 0.016;

    if ((key_event = key_read()) != NO_EVENT)
    {
      switch (key_event)
      {
        case LIGHT_UP:
          light.chg_pos(0, 0, inc, 0);
          lt_pos.y += inc;
          break;
        case LIGHT_DOWN:
          light.chg_pos(0, 0, -inc, 0);
          lt_pos.y -= inc;
          break;
        case LIGHT_EAST:
          light.chg_pos(0, inc, 0, 0);
          lt_pos.x += inc;
          break;
        case LIGHT_WEST:
          light.chg_pos(0, -inc, 0, 0);
          lt_pos.x -= inc;
          break;

        case MORE_RED:
          if (lt_color.r < 1.0)
          {
            light.chg_color(0, c_inc, 0, 0);
            lt_color.r += c_inc;
          }
          display_color_values(0);
          break;
        case MORE_GREEN:
          if (lt_color.g < 1.0)
          {
            light.chg_color(0, 0, c_inc, 0);
            lt_color.g += c_inc;
          }
          display_color_values(1);
          break;
        case MORE_BLUE:
          if (lt_color.b < 1.0)
          {
            light.chg_color(0, 0, 0, c_inc);
            lt_color.b += c_inc;
          }
          display_color_values(2);
          break;
        case LESS_RED:
          if (lt_color.r > 0.01)
          {
            light.chg_color(0, -c_inc, 0, 0);
            lt_color.r -= c_inc;
          }
          display_color_values(0);
          break;
        case LESS_GREEN:
          if (lt_color.g > 0.01)
          {
            light.chg_color(0, 0, -c_inc, 0);
            lt_color.g -= c_inc;
          }
          display_color_values(1);
          break;
        case LESS_BLUE:
          if (lt_color.b > 0.01)
          {
            light.chg_color(0, 0, 0, -c_inc);
            lt_color.b -= c_inc;
          }
          display_color_values(2);
          break;

        case EXIT:
          done = 1;
      }

      // Recalculate lighting
      light.determine(s);
    }

    // Draw the current surface
    draw_scene(ON);
    graphics.chg_visual_page();
    delay(20);
  }

  // Return to text mode
  graphics.off();
}

//-------------------------------------------------------------------------
// Draw the surface and the light source
//-------------------------------------------------------------------------
void draw_scene(int state)
{
  if (state == ON)
  {
    if (lt_pos.y > 0)
    {
      display_surface();
      display_light();
    }
    else
    {
      display_light();
      display_surface();
    }
  }
  else
    graphics.clear_port(VIEWPORT, BLACK);
}

//-------------------------------------------------------------------------
// Draw the light source
//-------------------------------------------------------------------------
void display_light()
{
  struct palettetype pal;
  getpalette(&pal);

  // Draw the light source
  viewer.translate(&lt_pos, &lt_source);

  // Set the BLUE palette to the current shade
  setrgbpalette(pal.colors[BLUE], lt_color.r*63, 
                lt_color.g*63, lt_color.b*63);
  putpixel(lt_source.x, lt_source.y, BLUE);
  putpixel(lt_source.x-1, lt_source.y, BLUE);
  putpixel(lt_source.x+1, lt_source.y, BLUE);
  putpixel(lt_source.x, lt_source.y-1, BLUE);
  putpixel(lt_source.x, lt_source.y+1, BLUE);
}

//-------------------------------------------------------------------------
// Display the square at its current position, from our current
//  point of view, using the supplied color
//-------------------------------------------------------------------------
void display_surface()
{
  int i, j;
  struct palettetype pal;
  getpalette(&pal);

  // Calculate the 3-D location of each point
  for (i = 0; i < 4; i++)
  {
    d_point temp;
    viewer.rotate(&square[i].pos, &temp);
    viewer.project(&temp, &square[i].pixel);
  }

  // Fill the square with a solid color
  for (j = 0; j < 4; j++)
    memcpy(&poly[j * 2], &square[j].pixel, 4);
  setfillstyle(SOLID_FILL, WHITE);  // Fill color
  setcolor(BLACK);                  // Outline color

  // Set the WHITE palette to the current shade
  setrgbpalette(pal.colors[WHITE], my_surface.lighting.r*63, 
                my_surface.lighting.g*63, my_surface.lighting.b*63);
  fillpoly(4, poly);
}

//------------------------------------------------------------------------
// Determine the coefficients of a plane equation: ax + by + cz + d = 0
//  returning: norm (a,b,c) and scal (d).
//------------------------------------------------------------------------
void define_surface(d_plane *p, d_point *base, d_point *v1, d_point *v2)
{
  // Copy the defining d_points to the plane structure
  memcpy(&p->base, base, sizeof(d_point));

  // Calculate direction vectors for any two lines on the plane
  d_point vec1 = vect.sub(base, v1);
  d_point vec2 = vect.sub(base, v2);

  // Coefficients: a b and c will be the three coordinates
  //  of the surface normal
  p->normal = vect.cross(&vec1, &vec2);
  vect.normalize(&p->normal);

  // Determine coefficient d using the normal and the base d_point
  p->scalar = vect.dot(&p->normal, base);
}

//------------------------------------------------------------------------
// Keyboard processor for the about window
//------------------------------------------------------------------------
long key_read()
{
  long ret_event = NO_EVENT;
  long the_key = get_key();

  // Continue if there is a key to analyze
  if (the_key != NO_KEY)
  {
    int asc_key = the_key < 255 ? (int)the_key : 0;
   
    if (asc_key)
    {
      asc_key = toupper(asc_key);
      switch(asc_key)
      {
        case 'Q':
          ret_event = EXIT;
          break;
      }
    }
    else
    {
      switch (the_key)
      {
        case F1:
          ret_event = LESS_RED;
          break;
        case F2:
          ret_event = MORE_RED;
          break;
        case F3:
          ret_event = LESS_GREEN;
          break;
        case F4:
          ret_event = MORE_GREEN;
          break;
        case F5:
          ret_event = LESS_BLUE;
          break;
        case F6:
          ret_event = MORE_BLUE;
          break;

        case LEFT:
        case KP_LEFT:
        case S_KP_LEFT:
          ret_event = LIGHT_WEST;
          break;
        case RIGHT:
        case KP_RIGHT:
        case S_KP_RIGHT:
          ret_event = LIGHT_EAST;
          break;
        case UP:
        case KP_UP:
        case S_KP_UP:
          ret_event = LIGHT_UP;
          break;
        case DOWN:
        case KP_DOWN:
        case S_KP_DOWN:
          ret_event = LIGHT_DOWN;
          break;

        case ESC:
          ret_event = EXIT;
          break;
      }
    }
  }
 
  keyboard.key_code = NO_KEY;
  return ret_event;
}

//-------------------------------------------------------------------------
// If no key is waiting to be analyzed, then check for a new one.
//-------------------------------------------------------------------------
long get_key()
{
  long the_key = keyboard.last();
  
  if (the_key == NO_KEY && keyboard.check())
    the_key = keyboard.read();

  return the_key;
}

//-------------------------------------------------------------------------
// Display help text
//-------------------------------------------------------------------------
void show_text(int which)
{
  int r_y = 14;
  int g_y = r_y + 12;
  int b_y = g_y + 12;
  int tx = 526;
  int cx = 414;

  if (which == -1)
  {
    graphics.print(10, 0, LIGHTGRAY, "Lighting Demo - XYZ++");
    graphics.print(10, 14, CYAN, "Left/Right keys move light Away/Towards Center");
    graphics.print(10, 26, CYAN, "Up/Down keys move light Higher/Lower");
    graphics.print(cx-4, 0, LIGHTGRAY, "Light Color   Sub  Add");
    graphics.print(tx, r_y, LIGHTRED, "F1   F2");
    graphics.print(tx, g_y, LIGHTGREEN, "F3   F4");
    graphics.print(tx, b_y, LIGHTBLUE, "F5   F6");
  }
  if (which == -1 || which == 0)
    graphics.print(cx, r_y, LIGHTRED, "R: %5.5f", lt_color.r);
  if (which == -1 || which == 1)
    graphics.print(cx, g_y, LIGHTGREEN, "G: %5.5f", lt_color.g);
  if (which == -1 || which == 2)
    graphics.print(cx, b_y, LIGHTBLUE, "B: %5.5f", lt_color.b);
}

//-------------------------------------------------------------------------
// Display current color values
//-------------------------------------------------------------------------
void display_color_values(int which)
{
  graphics.set_port(HELPPORT);
  graphics.chg_active_page();
  show_text(which);
  graphics.chg_active_page();
  show_text(which);
  graphics.set_port(VIEWPORT);
  viewer.set_zoom(70);
}

