//----------------------------------------------
// EG_PANEL -- XYZ++ example program
//----------------------------------------------
// EGA control panel example
//----------------------------------------------
// Copyright (C) 1993, Nicholas Centanni
//----------------------------------------------
#include <graphics.h>
#include <op_graph.h>
#include <op_panel.h>
#include <op_mouse.h>
#include <op_key.h>
#include "eg_panel.h"

Graphics graphics;
Mouse mouse(&graphics);
Keyboard keyboard;
EGAPanel panel(&graphics, &mouse, &keyboard);

int x = 130;
int y = 180;
int width = 380;
int height = 70;
long led_val1;
double led_val2;
char prompt_str[16];
int get_events(void);

//---------------------------------------------------------------------------
// Program MAIN
//---------------------------------------------------------------------------
void main()
{
  // Enter graphics mode
  registerbgidriver(EGAVGA_driver);
  registerbgifont(small_font);
  graphics.on(EGA, EGAHI, "");        
  graphics.define_port(0, 0, 0, 640, 350);   // Define a view rectangle
  graphics.set_port(0);                      // Assign object to a port

  // Define the font to be used
  settextstyle(SMALL_FONT, HORIZ_DIR, 4);
  graphics.set_txt_background(EG_LT_GRAY);

  // Set our custom EGA palette
  setallpalette(&pal);

  // Display the panel
  panel.define();
  panel.display();

  // Look for a mouse
  mouse.activate();
  if (mouse.active)
  {
    mouse.set_method(STD_DRAW);
    mouse.moveto(320, 300);
    mouse.show();
  }

  // Enter the control panel
  while (get_events());

  // Return to text mode
  if (mouse.active)
   mouse.activate();
  graphics.off();
}

//-----------------------------------------------------------------------
// Event loop
//-----------------------------------------------------------------------
int get_events()
{
  int event;
  int do_event = FALSE;

  // Check for a new event
  if ((event = panel.events()) != NO_EVENT)
    do_event = TRUE;
  else
  {
    led_val2 += 0.0001;
    panel.led[LED_2]->print(led_val2);
    delay(30);
  }

  // Check for program end
  if (event == QUIT_BTN)
    return 0;

  // Process events in the panel
  if (do_event)
  {
    switch (event)
    {
      case UP_BTN:
        if (led_val1 < 100)
        {
          led_val1++;
          panel.led[LED_1]->print(led_val1);
          delay(30);
        }
        break;

      case DN_BTN:
        if (led_val1 > 0)
        {
          led_val1--;
          panel.led[LED_1]->print(led_val1);
          delay(30);
        }
        break;

      case RESET_BTN:
        led_val2 = 0;
        panel.led[LED_2]->print(led_val2);
        delay(500);
        break;
    }
  }

  // Clear the keyboard events and keyboard buffer
  panel.key_event = NO_EVENT;
  panel.clear_key();

  return TRUE;
}
