//----------------------------------------------
// EG_PANEL -- XYZ++ example program
//----------------------------------------------
// EGA control panel member functions
//----------------------------------------------
// Copyright (C) 1993, Nicholas Centanni
//----------------------------------------------
#include <ctype.h>
#include <op_panel.h>
#include "eg_panel.h"

//----------------------------------------------
// This function will be called to set up
// panel components
//----------------------------------------------
void EGAPanel::define()
{
  int cy = 20;

  // Define the LED controls
  add_led(led_map1, x + 10, y + cy, EG_RED, 6);
  led[LED_1]->set_value(led_val1);
  led[LED_1]->set_port_no(0);
  led[LED_1]->position_text(4, 2);
  add_led(led_map2, x + 90, y + cy, EG_CYAN, 9);
  led[LED_2]->set_fixed(4, 10000);
  led[LED_2]->set_value(led_val2);
  led[LED_2]->set_port_no(0);

  // Define the buttons
  int lx = led[LED_1]->x2;
  int ly = led[LED_1]->y1;
  add_BMPbutton(lx + 2, ly - 7, 16, 12, u_up, u_dn, 0L, SNAP_UP_BTN);
  add_BMPbutton(lx + 2, ly + 9, 16, 12, d_up, d_dn, 0L, SNAP_UP_BTN);
  button[UP_BTN]->set_port_no(0);
  button[DN_BTN]->set_port_no(0);
  lx = led[LED_2]->x1;
  ly = led[LED_2]->y2;
  add_BGIbutton(x + width - 50, y + cy + 1, 32, 15, "Quit", 5, 1, EG_BLACK, EG_LT_GRAY,
                EG_DK_GRAY, EG_WHITE, SMALL_FONT, 4, SNAP_UP_BTN);
  add_BGIbutton(lx + 16, ly + 5, 42, 15, "Reset", 6, 1, EG_BLACK, EG_LT_GRAY,
                EG_DK_GRAY, EG_WHITE, SMALL_FONT, 4, SNAP_UP_BTN);

  // Define a text prompt
  add_prompt(x + 180, y + cy + 2, EG_LT_GREEN, EG_DK_GREEN, 
             EG_DK_GREEN, prompt_str, 20);
}

//----------------------------------------------
// This function will be called to display the
// panel on screen.
//----------------------------------------------
void EGAPanel::display()
{
  // Draw the panel background
  setfillstyle(SOLID_FILL, EG_LT_GRAY);
  bar(x, y, x + width, y + height);
  setcolor(EG_WHITE);
  setlinestyle(SOLID_LINE, SOLID_LINE, 3);
  moveto(x, y);
  lineto(x + width, y);
  lineto(x + width, y + height);
  lineto(x, y + height);
  lineto(x, y);

  // Carve in a beveled area for the prompt
  int px = prompt[0]->x1 - 7;
  int py = prompt[0]->y1 - 3;
  int pwidth = prompt[0]->x2 - prompt[0]->x1 + 15;
  int pheight = prompt[0]->y2 - prompt[0]->y1 + 8;
  graphics.inset_bevel(px, py, pwidth, pheight, EG_LT_GRAY, 
                       EG_BLACK, EG_DK_GRAY, EG_WHITE);

  // Draw the controls
  draw();

  // Start the prompt
  curr_prompt = 0;
  prompt[curr_prompt]->start_prompt();

  // Draw labels
  Led *l = led[LED_1];
  graphics.set_txt_background(EG_LT_GRAY);
  graphics.print(l->x1 + 3, l->y1 - 12, EG_BLACK, "Integer");
  l = led[LED_2];
  graphics.print(l->x1 + 23, l->y1 - 12, EG_BLACK, "Double");
  Prompt *p = prompt[0];
  graphics.print(p->x1 - 1, l->y1 - 12, EG_BLACK, "Text Prompt");
  graphics.set_txt_background(EG_BLACK);
}

//----------------------------------------------
// This function does custom filtering of
// keystrokes received by the panel.
//----------------------------------------------
long EGAPanel::key_read(void)
{
  int ret_event = NO_EVENT;
  long the_key = get_key();
 
  // Continue if there is a key to analyze
  if (the_key != NO_KEY)
  {
    int asc_key = the_key < 255 ? (int)the_key : 0;
  
    if (asc_key != 0)
    {
      asc_key = toupper(asc_key);

      switch(asc_key)
      {
        case '1':
          ret_event = QUIT_BTN;
          break;

        default:
          ret_event = NEW_KEY + the_key;
          break;
      }
    }

    if (ret_event == NO_EVENT)
    {
      switch (the_key)
      {
        case UP:
        case S_KP_UP:
        case KP_UP:
          ret_event = UP_BTN;
          break;
        case DOWN:
        case S_KP_DOWN:
        case KP_DOWN:
          ret_event = DN_BTN;
          break;
        case ESC:
        case ENTER:
        case KP_ENTER:
          ret_event = QUIT_BTN;
          break;
        default:
          ret_event = NEW_KEY + the_key;
          break;
      }
    }
  }

  if (ret_event != NO_EVENT)
    clear_key();

  return ret_event;
}


