//------------------------------------------------------------------------
// Star field member functions
//------------------------------------------------------------------------
#include <stdlib.h>
#include <string.h>
#include "sp_ver.h"

//--------------------------------------------------------------------------
// Space Constructor
//--------------------------------------------------------------------------
Space::Space()
{
  stars = new star[MAX_STARS];
  tot_stars = MAX_STARS;
  origin.x = origin.y = 0L;
  origin.z = ORIGIN_Z;

  // Create the first group of stars
  new_stars();
}

//--------------------------------------------------------------------------
// Space Destructor
//--------------------------------------------------------------------------
Space::~Space()
{
  if (stars != 0L)
    delete stars;
}

//--------------------------------------------------------------------------
// Place the first random grouping of stars in space
//--------------------------------------------------------------------------
void Space::new_stars()
{
  randomize();

  for (int i = 0; i < tot_stars; i++)
  {
    star *s = &stars[i];

    // Determine a random pitch and heading
    int pitch = random(TOT_ANGLES);
    int heading = random(TOT_ANGLES);
    long dist = MIN_DIST + (long)random(BIRTH_ZONE);

    // Start at the origin
    viewer.move(&origin, &s->pos, heading, pitch, dist);
    s->color = random(15) + 1;
  }
}

//--------------------------------------------------------------------------
// Refresh an expired star
//--------------------------------------------------------------------------
void Space::new_star(int which)
{
  if (which < tot_stars)
    stars[which].pos.z = origin.z - random(REFRESH_ZONE);
}

//--------------------------------------------------------------------------
// Move the stars
//--------------------------------------------------------------------------
void Space::move(int how_far)
{
  for (int i = 0; i < tot_stars; i++)
  {
    if (stars[i].pos.z > 0L)
      new_star(i);
    else
      stars[i].pos.z += how_far;
  }
}

//--------------------------------------------------------------------------
// Space DRAW function
//--------------------------------------------------------------------------
void Space::draw(int mode)
{
  if (mode == ON)
  {
    for (int i = 0; i < tot_stars; i++)
    {
      star *s = &stars[i];

      viewer.translate(&s->pos, &s->pixel);
      if (s->pixel.visible)
      {
        fg_setcolor(s->color);
        fg_point(s->pixel.x, s->pixel.y);
      }
    }
  }
  else
    fg_erase();
}

