//------------------------------------------------------------------------
// Star field animation -- XYZ++ sample application
//------------------------------------------------------------------------
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include "sp_ver.h"

Viewer viewer;
Space space;
int old_mode;

// Transitional code necessary when using a 3rd party graphics library
int active_page;
int visual_page;

void swap_active_page()
{
  fg_setpage(active_page = !active_page);
}

void swap_visual_page()
{
  fg_setvpage(visual_page = !visual_page);
}

void set_port(int lx, int ly, int hx, int hy)
{
  Lox = lx; Hix = hx;
  Loy = ly; Hiy = hy;
}

//--------------------------------------------------------------------------
// Program MAIN
//--------------------------------------------------------------------------
void main()
{
  // Bail out now if there was not enough memory for the starfield
  if (space.stars == 0L)
  {
    printf("Not enough memory to continue.\n");
    exit(0);
  }

  old_mode = fg_getmode();       // Set up the graphics mode
  fg_setmode(16);
  set_port(0, 0, 640, 350);      // Set a viewing port
  fg_waitvr(0);                  // Disable the vertical retrace wait

  viewer.set_zoom(80);
  viewer.set_position(0, 0, 0);
  viewer.set_center(0, 0, -2000);
  viewer.view_center();
  viewer.new_view();

  // Draw the starfield on both pages
  swap_active_page();
  space.draw(ON);
  swap_active_page();
  space.draw(ON);

  while (!kbhit())
  {
    // Draw the starfield on the active page
    swap_active_page();
    space.draw(OFF);
    space.move(20);
    space.draw(ON);
    swap_visual_page();

    delay(20);
  }
  getch();

  // Return to text mode
  fg_setmode(old_mode);
  fg_reset();
}

//--------------------------------------------------------------------------
// Exit the program if the host system is inadequate
//--------------------------------------------------------------------------
void bail(int cause)
{
  fg_setmode(old_mode);

  if (cause == MEM_ERR)
    printf("Not enough memory to run.\n");
  else if (cause == BGI_ERR)
    printf("Could not start BGI graphics mode.\n");

  exit(-1);
}
