//----------------------------------------------
// SQUARE -- XYZ++ example program
//----------------------------------------------
// Draws a square plane.
//----------------------------------------------
// Copyright (C) 1993, Nicholas Centanni
//----------------------------------------------
#include <graphics.h>
#include <dos.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
#include <op_graph.h>
#include <op_view.h>

Graphics graphics;       // Declare a graphics object
Viewer viewer;           // Declare a 3-D viewer object

// Define the structure that describes each corner point
struct square_vertex
{
  l_point world;         // Point's world coordinate
  i_screen pixel;        // Screen coordinates - one for each video page
};

// Declare an array of four vertices in space.
//  x = the number of units to the EAST of zero
//  y = the number of units in HEIGHT above zero
//  z = the number of units SOUTH of zero
struct square_vertex square[] =
{
  //- x   y   z -//
  {  20, 20, 20 },
  {  60, 20, 20 },
  {  60, 20, 60 },
  {  20, 20, 60 },
};

#define VIEWPORT   0
int poly[4];

// Supporting functions
void draw_square(void);
void display(int color);

//-------------------------------------------------------------------------
// Program main
//-------------------------------------------------------------------------
void main()
{
  // Start graphics
  registerbgidriver(EGAVGA_driver);
  graphics.on(EGA, EGAHI, "");
  delay(800);  // Give auto-sync screens a chance to settle down

  // Define a viewing window - there MUST be at least one
  graphics.define_port(VIEWPORT, 0, 0, getmaxx(), getmaxy());
  graphics.set_port(VIEWPORT);

  // The 3-D zoom setting MUST take place AFTER
  // the above calls to define_port and set_port
  viewer.set_zoom(100);

  // Draw the object
  draw_square();
  getch();

  // Return to text mode
  graphics.off();
}

//-------------------------------------------------------------------------
// Move past the object in space
//-------------------------------------------------------------------------
void draw_square()
{
  // Place the viewer at a position in space: x, y, z
  viewer.set_position(40, 50, 100);

  // Set the center of interest to be the middle of the square: x, y, z
  viewer.set_center(40, 20, 40);
  viewer.view_center();  // Look at the center of interest

  // Display the object
  display(GREEN);
}

//-------------------------------------------------------------------------
// Display the square at its current position, from our current
//  point of view, using the supplied color
//-------------------------------------------------------------------------
void display(int color)
{
  int i, j;

  // Calculate the viewing matrix for this angle of view
  viewer.new_view();

  // Calculate the 3-D location of each point
  for (i = 0; i < 4; i++)
    viewer.translate(&square[i].world, &square[i].pixel);

  // Fill the square with a solid color
  for (j = 0; j < 4; j++)
    memcpy(&poly[j * 2], &square[j].pixel, 4);
  setfillstyle(SOLID_FILL, color);  // Fill color
  setcolor(BLACK);                  // Outline color
  fillpoly(4, poly);
}
