//----------------------------------------------
// SQ_MOVE -- XYZ++ example program
//----------------------------------------------
// Animates viewer moving in relation to a 
// square plane.
//----------------------------------------------
// Copyright (C) 1993, Nicholas Centanni
//----------------------------------------------
#include <graphics.h>
#include <dos.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
#include <op_graph.h>
#include <op_view.h>

Graphics graphics;       // Declare a graphics object
Viewer viewer;           // Declare a 3-D viewer object

// Define the structure that
// describes each corner point
struct square_vertex
{
  l_point world;         // Point's world coordinate
  i_screen pix[2];       // Screen coordinates - one for each video page
};

// Declare an array of four vertices in space.
//  x = the number of units to the EAST of zero
//  y = the number of units in HEIGHT above zero
//  z = the number of units SOUTH of zero
struct square_vertex square[] =
{
  //- x   y   z -//
  {  20, 20, 20 },
  {  60, 20, 20 },
  {  60, 20, 60 },
  {  20, 20, 60 },
};

// Main viewing window
#define VIEWPORT   0

// Polygon structure for the use of the
// BGI fillpoly() function
int poly[4];

// Supporting functions
void display(int color);
void animate(void);

//-------------------------------------------------------------------------
// Program main
//-------------------------------------------------------------------------
void main()
{
  // Start graphics
  registerbgidriver(EGAVGA_driver);
  graphics.on(EGA, EGAHI, "");
  delay(800);  // Give auto-sync screens a chance to settle down

  // Define a viewing window - there MUST be at least one
  graphics.define_port(VIEWPORT, 0, 0, getmaxx(), getmaxy(), PAGING_ON);
  graphics.set_port(VIEWPORT);
  graphics.paging_on();  // Enable animation

  // The 3-D zoom setting MUST take place AFTER
  // the above calls to define_port and set_port
  viewer.set_zoom(100);

  // Repeatedly move through space, past the object
  animate();

  // Return to text mode
  graphics.off();
}

//-------------------------------------------------------------------------
// Move past the object in space
//-------------------------------------------------------------------------
void animate()
{
  int obj_color = GREEN;
  int starting_x = 40;
  int starting_y = 50;
  int starting_z = 100;
  int direction = -1;
  int x_min = -120;
  int x_max = 200;

  // Place the viewer at a position in space: x, y, z
  viewer.set_position(starting_x, starting_y, starting_z);

  // Set the center of interest to be the middle of the square: x, y, z
  viewer.set_center(40, 20, 40);
  viewer.view_center();  // Look at the center of interest

  // Display the object
  graphics.chg_active_page();
  display(obj_color);
  graphics.chg_visual_page();

  // Continuously move the viewpoint
  while (!kbhit())
  {
    // Erase the object
    graphics.chg_active_page();
    display(BLACK);

    // Change the viewer position
    viewer.move_west(6 * direction);
    viewer.move_up(direction * 3);

    // Check for direction reversal
    if (viewer.get_x() < x_min)
      direction = -1;
    else if (viewer.get_x() > x_max)
      direction = 1;

    // Aim the viewer towards the center of interest
    viewer.view_center();

    // Redisplay the square from the new angle
    display(obj_color);
    graphics.chg_visual_page();

    // Slow it down for display smoothness
    delay(30);
  }

  // Read the exit keystroke so it
  // doesn't pass on to DOS
  getch();
}

//-------------------------------------------------------------------------
// Display the square at its current position, from our current
//  point of view.
//-------------------------------------------------------------------------
void display(int color)
{
  int i, j;
  int page = graphics.active_page;

  // Only calculate the position when drawing the object
  if (color != BLACK)
  {
    // Calculate the viewing matrix for this angle of view
    viewer.new_view();

    // Calculate the 3-D location of each point
    for (i = 0; i < 4; i++)
      viewer.translate(&square[i].world, &square[i].pix[page]);
  }

  // Create the poly array
  for (j = 0; j < 4; j++)
    memcpy(&poly[j * 2], &square[j].pix[page], 4);

  setfillstyle(SOLID_FILL, color);  // Fill color
  setcolor(BLACK);                  // Outline color

  // Fill the square
  fillpoly(4, poly);
}

