//----------------------------------------------
// W_CUBE -- XYZ++ example program
//----------------------------------------------
// Animates a wire_frame cube.
//----------------------------------------------
// Copyright (C) 1993, Nicholas Centanni
//----------------------------------------------
#include <graphics.h>
#include <dos.h>
#include <conio.h>
#include <stdlib.h>
#include <op_graph.h>
#include <op_view.h>

#define VIEWPORT   0
#define UP         1
#define DOWN       2

Graphics graphics;       // Declare a graphics object
Viewer viewer;           // Declare a 3-D viewer object

// Define the cube size
#define SIZE      60

// Define the cube center coordinates
#define CX        10
#define CY        10
#define CZ        10

// Define a vertex for the center of the cube
l_vertex center = { CX, CY, CZ };

// Define the cube
l_vertex cube[] =
{
  { CX - SIZE, CY - SIZE, CZ - SIZE },    // Bottom four vertices
  { CX + SIZE, CY - SIZE, CZ - SIZE },
  { CX + SIZE, CY - SIZE, CZ + SIZE },
  { CX - SIZE, CY - SIZE, CZ + SIZE },

  { CX - SIZE, CY + SIZE, CZ - SIZE },    // Top four vertices
  { CX + SIZE, CY + SIZE, CZ - SIZE },
  { CX + SIZE, CY + SIZE, CZ + SIZE },
  { CX - SIZE, CY + SIZE, CZ + SIZE },
};

// An array of 12 line descriptions will hold
// the index in the above cube structure of the
// endpoints for all of the edge lines.
struct cube_edge
{
  int en1;    // Endpoint 1
  int en2;    // Endpoint 2
};

// Initialize the cube edge descriptors
struct cube_edge edge[] =
{
  { 0, 1 }, { 1, 2 }, { 2, 3 }, { 3, 0 },   // Base lines
  { 4, 5 }, { 5, 6 }, { 6, 7 }, { 7, 4 },   // Top lines
  { 0, 4 }, { 1, 5 }, { 2, 6 }, { 3, 7 },   // Vertical edge lines
};

// Supporting functions
void display(int color);
void animate(void);

//-------------------------------------------------------------------------
// Program main
//-------------------------------------------------------------------------
void main()
{
  // Start graphics
  registerbgidriver(EGAVGA_driver);
  graphics.on(VGA, VGAHI, "");
  delay(800);  // Give auto-sync screens a chance to settle down

  // Define a viewing window - there MUST be at least one
  graphics.define_port(VIEWPORT, 0, 0, getmaxx(), getmaxy());
  graphics.set_port(VIEWPORT);

  // The 3-D zoom setting MUST take place AFTER
  // the above calls to define_port and set_port
  viewer.set_zoom(100);

  // Run the animation until a key is pressed
  animate();

  // Return to text mode
  graphics.off();
}

//-------------------------------------------------------------------------
// Move past the object in space
//-------------------------------------------------------------------------
void animate()
{
  int obj_color = LIGHTCYAN;
  int rotate_chg = 20;
  int vertical_chg = 5;
  int max_height = 200;
  int direction = DOWN;

  // Place the viewer at a position in space: x, y, z
  viewer.set_position(300, 100, 200);

  // Set the center of interest to be the middle of the cube
  viewer.set_center(center.pos.x, center.pos.y, center.pos.z);
  viewer.view_center();  // Aim the viewer at the center

  // Display the object
  display(obj_color);

  // Animate
  while (!kbhit())
  {
    // Erase the object
    display(BLACK);

    // Change the viewer position
    viewer.orbit(rotate_chg);
    viewer.chg_heading(rotate_chg);
    if (direction == DOWN)
      viewer.move_down(vertical_chg);
    else
      viewer.move_up(vertical_chg);

    // Change directions at ground level and at max_height
    if (viewer.get_y() <= 0)
      direction = UP;
    else if (viewer.get_y() >= max_height)
      direction = DOWN;

    // Redisplay the object from the new angle
    display(obj_color);

    // Move slowly
    delay(40);
  }

  // Read the exit keystroke so it 
  // doesn't pass on to DOS
  getch();
}

//-------------------------------------------------------------------------
// Display the cube at its current position, 
// from the current point of view.
//-------------------------------------------------------------------------
void display(int color)
{
  int i;

  // Only calculate the vertices when drawing the object
  if (color != BLACK)
  {
    // Calculate the viewing matrix for this angle of view
    viewer.new_view();

    // Calculate the 3-D location of each point
    for (i = 0; i < 8; i++)
      viewer.translate(&cube[i].pos, &cube[i].pixel);
  }

  // The PIXEL component of each vertex now represents its
  // correct position on the screen -- Draw the cube edge lines
  setcolor(color);
  for (i = 0; i < 12; i++)
  {
    int e1 = edge[i].en1;
    int e2 = edge[i].en2;

    line(cube[e1].pixel.x, cube[e1].pixel.y,
         cube[e2].pixel.x, cube[e2].pixel.y);
  }
}

