//--------------------------------------------------
// DOS/BGI/ASM Graphics interface class
//--------------------------------------------------
// $Revision:   1.27  $
// $Date:   21 Apr 1994 15:20:06  $
//--------------------------------------------------
#ifndef __OP_GRAPH_H
#define __OP_GRAPH_H
#ifndef __GRAPHICS_H
#include <graphics.h>
#endif
#ifndef __OP_TYPES_H
#include <op_types.h>
#endif

//---------------------------------
// Constants
//---------------------------------
#define COPY_BITS           0
#define XOR_BITS            1
#define SCREEN_BYTES        80
#define CLIP_BYTES          SCREEN_BYTES * 3
#define MAX_PATH            40
#define SCREEN_SEG          0xA000
#define SCREEN_PIXWIDTH     640   // in pixels
#define SCREEN_WIDTH        80    // in bytes
#define SPLIT_ROW           339
#define LO_LINES            141
#define HI_LINES            SPLIT_ROW
#define PAGE_0             (LO_LINES * SCREEN_WIDTH)
#define PAGE_1            ((LO_LINES + HI_LINES) * SCREEN_WIDTH)
#define CRTC_INDEX          0x3D4 // CRT Controller Index register
#define CRTC_DATA           0x3D5 // CRT Controller Data register
#define OVERFLOW            0x07
#define MAX_SCAN            0x09
#define LINE_COMPARE        0x18
#define MAX_PORTS           10
#define FULL_SCREEN         0

// Paging modes
#define OFS_ADR             0
#define SEG_ADR             1
#define BGI_PAGING          0
#define PR_PAGING           1

// Host graphics mode
#define SPLIT_OFF         0    
#define SPLIT_ON          1
#define PAGING_OFF        0
#define PAGING_ON         1

//---------------------------------
// Globals for assembly interface
//---------------------------------
extern int left_byte, right_byte;      // Window-edge byte offsets
extern char left_mask, right_mask;     // Window-edge bit offsets
extern int In_Lox, In_Loy,             //------------------
       In_Hix, In_Hiy;                 //
extern int Loy_1, Hiy_1,               //  Inner clipping
       Lox_1, Hix_1;                   //  boundries
extern int In_Lox_1, In_Loy_1,         //
       In_Hix_1, In_Hiy_1;             //------------------
extern unsigned int ScrnSeg;           // Video page Segment
extern unsigned int ScrnOfs;           // Video page Offset
extern int clip_zero;                  // Index of X = 0 in clip array
extern char clip_map[CLIP_BYTES];      // Clipping array for clip_bmp()
extern "C" clip_bmp(char *bmp, int x,  // Low-level bitmap display function
                    int y, int mode);  //  with HORZ and VERT clipping
extern "C" draw_bmp(char *bmp, int x,  // Low-level bitmap display function
                    int y, int mode);  //  with no clipping
extern "C" void show_page(unsigned int s_offset);  // Page flipper
extern "C" void set_page(int page);                // Vert_Retrace flipper
extern "C" void gr_reset(void);        // Reset ALL graphics registers
extern int tx1, ty1, tx2, ty2;         // Polygon FILL data
extern int tx3, ty3, tx4, ty4;         // |

// Screen paging
extern int visible_page, hidden_page;
extern unsigned int pages[2];

// Optimized LINE drawing function
extern "C" n_line(int x1, int y1, int x2, int y2, int color, int how);

// Optimized pixel plotting functions
extern "C" plot_1(int x, int y, int color);
extern "C" plot_4(int x, int y, int color);
extern "C" plot_5(int x, int y, int color1, int color2);

                        //-----------------------------------
struct port             // Viewing window screen coordinates
{                       //-----------------------------------
 int x1;                // Upper left
 int y1;                //
 int x2;                // Lower right
 int y2;                //
 int paged;             // Exists on a page-flipped region
 int split;             // Exists in a split_vga mode
};

struct a_char
{
 int width;
 char bitmap[56];
};

// List structure for polygon fillers
struct h_line
{
  unsigned st_loff;
  char st_lbit;
  unsigned en_loff;
  char en_lbit;
  unsigned st_roff;
  char st_rbit;
  unsigned en_roff;
  char en_rbit;
};
extern h_line edges[];
extern int array_index;

//-------------------------------------------------------------------------
// Graphics interface class
//-------------------------------------------------------------------------
class Graphics
{
 private:
  int gr_on;                    // TRUE when graphics are on
  int gdriver;                  // Current graphics driver
  int gmode;                    // Current graphics mode
  static char gpath[MAX_PATH];  // Current path of BGI drivers
  void set_max_pages(void);     // Records #of pages based on Graphics mode
  int max_pages;                // Results of the set_max_pages() call
  int pages_on;
  int clip_mask_ready;          // Records completion of bitmap clip array
  unsigned int pages[2];        // Page address array
  int screen_addressing;        // Determines type of paging to use
  int paging_system;            // Determines method of switching pages

 public:
  //----------------------------------------------------------------------
  // Initialization functions
  //----------------------------------------------------------------------
  Graphics(void) : gr_on(0), txt_background(0), clip_mask_ready(0) {}
  ~Graphics(void) {}
  void on(void);
  void on(char *gpth);
  void on(int gdrv, int gmod, char *gpth);
  void off(void);
  void reset(void) { gr_reset(); }

  //----------------------------------------------------------------------
  // Viewport system
  //----------------------------------------------------------------------
  int tot_ports;                    // Total ports currently defined
  port ports[MAX_PORTS];            // Pointer to an array of port records
  int curr_port;                    // Handle of current viewport
  int wx1, wy1, wx2, wy2;           // Current viewing rectangle
  int cx, cy;                       // Window center coordinates
  void define_port(int port_no, int x1, int y1,
                   int x2, int y2,
                   int paged = PAGING_OFF,
                   int split = SPLIT_OFF);   // Define a port record
  void set_port(int port_no);                // Set PORT for current output
  void set_port(int port_no, int color);     // Set PORT and border it
  void get_port(int port_no, port *p);       // Get current PORT dimensions
  void clear_port(int port_no, int color);   // Clear current PORT
  void border_port(int port_no);
  void border_port(int port_no, int color);
  int port_ofs(int port_no);                 // Calculated y_offset for port
  void inner_clip(int x1, int y1,
                  int x2, int y2);  // Define an inner clipping region

  //----------------------------------------------------------------------
  // Paging system
  //----------------------------------------------------------------------
  int active_page;                // Active Page for graphics OUTPUT
  int visual_page;                // Visual Page for graphics DISPLAY
  void set_active(void);
  void set_visual(void);
  void enable_split_screen(void);
  void reset_pages(void);
  void chg_active_page(void);
  void chg_visual_page(void);
  void set_active_to_visual(void);
  void set_visual_to_active(void);
  void set_active_to_split(void);
  void paging_on(int adr = SEG_ADR, int mode = BGI_PAGING);
  void paging_off(void) { pages_on = 0; }
  int paging(void) { return pages_on; }
  void clear_screen(void);
  void wait_retrace(void);

  //----------------------------------------------------------------------
  // Text output
  //----------------------------------------------------------------------
  int txt_background;             // Background color
  void blank(int xloc, int yloc, int char_length);
  void print(int xloc, int yloc, int color, char *fmt, ...);
  void set_txt_background(int color) { txt_background = color; }

  //----------------------------------------------------------------------
  // Bitmap output
  //----------------------------------------------------------------------
  void prepare_clip_map(void);
  void draw_bitmap(char *map, int x, int y, int how);
  void clip_bitmap(char *map, int x, int y, int how);
  void change_bitmap_color(char *bmp, int old_color, int new_color);

  //----------------------------------------------------------------------
  // Effects
  //----------------------------------------------------------------------
  void inset_bevel(int x1, int y1, int odd_width, int odd_height, 
                   int back_color, int inner_color, 
                   int dark_shade, int light_shade);

  //----------------------------------------------------------------------
  // Polygon Fillers
  //----------------------------------------------------------------------
  int dither_fill(s_point *v_list, int v_cnt, int color);
  int solid_fill(s_point *v_list, int v_cnt, int color);
};

//-------------------------------------------------------------------------
// Graphics Font class
//-------------------------------------------------------------------------
class Graphics_font
{
 private:
  Graphics *gr;
  a_char *chset;
  int chmax;
  int old_char_color;
  int old_back_color;
  int char_color, back_color;
  int ch_spacing;
  void change_color(int ch_index);

 public:
  Graphics_font(Graphics *dp, a_char *ac, int cmax);
  ~Graphics_font(void) {}
  void set_colors(int ch_color, int bk_color);
  void set_spacing(int sp) { ch_spacing = sp; }
  void print(char *str, int x, int y);
};
#endif
