//--------------------------------------------------
// Keyboard event analyzer
//--------------------------------------------------
// $Revision:   1.0  $
// $Date:   30 Mar 1993 18:20:02  $
//--------------------------------------------------
#ifndef __OP_KEY_H
#define __OP_KEY_H

#define NO_KEY 0L

//-----------------------------
// Keyboard class
//-----------------------------
class Keyboard
{
 private:
  unsigned int key_value;
  unsigned int key_status;

 public:
  unsigned long key_code;

  Keyboard() : key_code(NO_KEY), key_value(0), key_status(0) {}
  int check(void);
  long read(void);
  long last(void) { return key_code; }
};

// NON-SHIFTED key codes
#define ESC          0x00011B
#define F1           0x003B00
#define F2           0x003C00
#define F3           0x003D00
#define F4           0x003E00
#define F5           0x003F00
#define F6           0x004000
#define F7           0x004100
#define F8           0x004200
#define F9           0x004300
#define F10          0x004400
#define F11          0x008500
#define F12          0x008600
#define BACKSP       0x000E08
#define TAB          0x000F09
#define ENTER        0x001C0D
#define SPACE        0x003920
#define INS          0x0052E0
#define HOME         0x0047E0
#define PGUP         0x0049E0
#define DEL          0x0053E0
#define END          0x004FE0
#define PGDN         0x0051E0
#define UP           0x0048E0
#define LEFT         0x004BE0
#define DOWN         0x0050E0
#define RIGHT        0x004DE0
#define KP_ENTER     0x00E00D
#define KP_HOME      0x004700
#define KP_UP        0x004800
#define KP_PGUP      0x004900
#define KP_LEFT      0x004B00
#define KP_CENTER    0x004C00
#define KP_RIGHT     0x004D00
#define KP_END       0x004F00
#define KP_DOWN      0x005000
#define KP_PGDN      0x005100
#define KP_INS       0x005200
#define KP_DEL       0x005300

// SHIFT key codes
#define S_ESC        0x04011B
#define S_F1         0x045400
#define S_F2         0x045500
#define S_F3         0x045600
#define S_F4         0x045700
#define S_F5         0x045800
#define S_F6         0x045900
#define S_F7         0x045A00
#define S_F8         0x045B00
#define S_F9         0x045C00
#define S_F10        0x045D00
#define S_F11        0x048700
#define S_F12        0x048800
#define S_BACKSP     0x040E08
#define S_TAB        0x040F00
#define S_ENTER      0x041C0D
#define S_SPACE      0x043920
#define S_INS        0x0452E0
#define S_HOME       0x0447E0
#define S_PGUP       0x0449E0
#define S_DEL        0x0453E0
#define S_END        0x044FE0
#define S_PGDN       0x0451E0
#define S_UP         0x0448E0
#define S_LEFT       0x044BE0
#define S_DOWN       0x0450E0
#define S_RIGHT      0x044DE0
#define S_KP_ENTER   0x04E00D
#define S_KP_HOME    0x044700
#define S_KP_UP      0x044800
#define S_KP_PGUP    0x044900
#define S_KP_LEFT    0x044B00
#define S_KP_CENTER  0x044C00
#define S_KP_RIGHT   0x044D00
#define S_KP_END     0x044F00
#define S_KP_DOWN    0x045000
#define S_KP_PGDN    0x045100
#define S_KP_INS     0x045200
#define S_KP_DEL     0x045300

// CTRL key codes
#define C_ESC        0x08011B
#define C_F1         0x085E00
#define C_F2         0x085F00
#define C_F3         0x086000
#define C_F4         0x086100
#define C_F5         0x086200
#define C_F6         0x086300
#define C_F7         0x086400
#define C_F8         0x086500
#define C_F9         0x086600
#define C_F10        0x086700
#define C_F11        0x088900
#define C_F12        0x088A00
#define C_2          0x080300
#define C_6          0x08071E
#define C_DASH       0x080C1F
#define C_BACKSP     0x080E7F
#define C_TAB        0x089400
#define C_Q          0x081011
#define C_W          0x081117
#define C_E          0x081205
#define C_R          0x081312
#define C_T          0x081414
#define C_Y          0x081519
#define C_U          0x081615
#define C_I          0x081709
#define C_O          0x08180F
#define C_P          0x081910
#define C_LBRACE     0x081A1B
#define C_RBRACE     0x081B1D
#define C_LINE       0x082B1C
#define C_A          0x081E01
#define C_S          0x081F13
#define C_D          0x082004
#define C_F          0x082106
#define C_G          0x082207
#define C_H          0x082308
#define C_J          0x08240A
#define C_K          0x08250B
#define C_L          0x08260C
#define C_ENTER      0x081C0A
#define C_Z          0x082C1A
#define C_X          0x082D18
#define C_C          0x082E03
#define C_V          0x082F16
#define C_B          0x083002 
#define C_N          0x08310E
#define C_M          0x08320D
#define C_SPACE      0x083920
#define C_INS        0x0892E0
#define C_HOME       0x0877E0
#define C_PGUP       0x0884E0
#define C_DEL        0x0893E0
#define C_END        0x0875E0
#define C_PGDN       0x0876E0
#define C_UP         0x088DE0
#define C_LEFT       0x0873E0
#define C_DOWN       0x0891E0
#define C_RIGHT      0x0874E0
#define C_KP_DIV     0x089500
#define C_KP_MULT    0x089600
#define C_KP_MINUS   0x088E00
#define C_KP_PLUS    0x089000
#define C_KP_ENTER   0x08E00A
#define C_KP_HOME    0x087700
#define C_KP_UP      0x088D00
#define C_KP_PGUP    0x088400
#define C_KP_LEFT    0x087300
#define C_KP_CENTER  0x088F00
#define C_KP_RIGHT   0x087400
#define C_KP_END     0x087500
#define C_KP_DOWN    0x089100
#define C_KP_PGDN    0x087600
#define C_KP_INS     0x089200
#define C_KP_DEL     0x089300

// ALT key codes
#define A_ESC        0x100100
#define A_F1         0x106800
#define A_F2         0x106900
#define A_F3         0x106A00
#define A_F4         0x106B00
#define A_F5         0x106C00
#define A_F6         0x106D00
#define A_F7         0x106E00
#define A_F8         0x106F00
#define A_F9         0x107000
#define A_F10        0x107100
#define A_F11        0x108B00
#define A_F12        0x108C00
#define A_TILDE      0x102900
#define A_1          0x107800
#define A_2          0x107900
#define A_3          0x107A00
#define A_4          0x107B00
#define A_5          0x107C00
#define A_6          0x107D00
#define A_7          0x107E00
#define A_8          0x107F00
#define A_9          0x108000
#define A_0          0x108100
#define A_DASH       0x108200
#define A_PLUS       0x108300
#define A_BACKSP     0x100E00
#define A_TAB        0x10A500
#define A_Q          0x101000
#define A_W          0x101100
#define A_E          0x101200
#define A_R          0x101300
#define A_T          0x101400
#define A_Y          0x101500
#define A_U          0x101600
#define A_I          0x101700
#define A_O          0x101800
#define A_P          0x101900
#define A_LBRACE     0x101A00
#define A_RBRACE     0x101B00
#define A_LINE       0x102B00
#define A_A          0x101E00
#define A_S          0x101F00
#define A_D          0x102000
#define A_F          0x102100
#define A_G          0x102200
#define A_H          0x102300
#define A_J          0x102400
#define A_K          0x102500
#define A_L          0x102600
#define A_COLON      0x102700
#define A_QUOTE      0x102800
#define A_ENTER      0x101C00
#define A_Z          0x102C00
#define A_X          0x102D00
#define A_C          0x102E00
#define A_V          0x102F00
#define A_B          0x103000
#define A_N          0x103100
#define A_M          0x103200
#define A_COMMA      0x103300
#define A_PERIOD     0x103400
#define A_QMARK      0x103500
#define A_SPACE      0x103920
#define A_INS        0x10A200
#define A_HOME       0x109700
#define A_PGUP       0x109900
#define A_DEL        0x10A300
#define A_END        0x109F00
#define A_PGDN       0x10A100
#define A_UP         0x109800
#define A_LEFT       0x109B00
#define A_DOWN       0x10A000
#define A_RIGHT      0x109D00
#define A_KP_DIV     0x10A400
#define A_KP_MULT    0x103700
#define A_KP_MINUS   0x104A00
#define A_KP_PLUS    0x104E00
#define A_KP_ENTER   0x10A600

// CTRL_ALT key codes
#define CA_ESC       0x180100
#define CA_F1        0x186800
#define CA_F2        0x186900
#define CA_F3        0x186A00
#define CA_F4        0x186B00
#define CA_F5        0x186C00
#define CA_F6        0x186D00
#define CA_F7        0x186E00
#define CA_F8        0x186F00
#define CA_F9        0x187000
#define CA_F10       0x187100
#define CA_F11       0x188B00
#define CA_F12       0x188C00
#define CA_TILDE     0x182900
#define CA_1         0x187800
#define CA_2         0x187900
#define CA_3         0x187A00
#define CA_4         0x187B00
#define CA_5         0x187C00
#define CA_6         0x187D00
#define CA_7         0x187E00
#define CA_8         0x187F00
#define CA_9         0x188000
#define CA_0         0x188100
#define CA_DASH      0x188200
#define CA_PLUS      0x188300
#define CA_BACKSP    0x180E00
#define CA_TAB       0x18A500
#define CA_Q         0x181000
#define CA_W         0x181100
#define CA_E         0x181200
#define CA_R         0x181300
#define CA_T         0x181400
#define CA_Y         0x181500
#define CA_U         0x181600
#define CA_I         0x181700
#define CA_O         0x181800
#define CA_P         0x181900
#define CA_LBRACE    0x181A00
#define CA_RBRACE    0x181B00
#define CA_LINE      0x182B00
#define CA_A         0x181E00
#define CA_S         0x181F00
#define CA_D         0x182000
#define CA_F         0x182100
#define CA_G         0x182200
#define CA_H         0x182300
#define CA_J         0x182400
#define CA_K         0x182500
#define CA_L         0x182600
#define CA_COLON     0x182700
#define CA_QUOTE     0x182800
#define CA_ENTER     0x181C00
#define CA_Z         0x182C00
#define CA_X         0x182D00
#define CA_C         0x182E00
#define CA_V         0x182F00
#define CA_B         0x183000
#define CA_N         0x183100
#define CA_M         0x183200
#define CA_COMMA     0x183300
#define CA_PERIOD    0x183400
#define CA_QMARK     0x183500
#define CA_SPACE     0x183920
#define CA_INS       0x18A200
#define CA_HOME      0x189700
#define CA_PGUP      0x189900
#define CA_END       0x189F00
#define CA_PGDN      0x18A100
#define CA_UP        0x189800
#define CA_LEFT      0x189B00
#define CA_DOWN      0x18A000
#define CA_RIGHT     0x189D00
#define CA_KP_DIV    0x18A400
#define CA_KP_MULT   0x183700
#define CA_KP_MINUS  0x184A00
#define CA_KP_PLUS   0x184E00
#define CA_KP_ENTER  0x18A600

// ALT_SHIFT key codes
#define AS_ESC       0x140100
#define AS_F1        0x146800
#define AS_F2        0x146900
#define AS_F3        0x146A00
#define AS_F4        0x146B00
#define AS_F5        0x146C00
#define AS_F6        0x146D00
#define AS_F7        0x146E00
#define AS_F8        0x146F00
#define AS_F9        0x147000
#define AS_F10       0x147100
#define AS_F11       0x148B00
#define AS_F12       0x148C00
#define AS_TILDE     0x142900
#define AS_1         0x147800
#define AS_2         0x147900
#define AS_3         0x147A00
#define AS_4         0x147B00
#define AS_5         0x147C00
#define AS_6         0x147D00
#define AS_7         0x147E00
#define AS_8         0x147F00
#define AS_9         0x148000
#define AS_0         0x148100
#define AS_DASH      0x148200
#define AS_PLUS      0x148300
#define AS_BACKSP    0x140E00
#define AS_TAB       0x14A500
#define AS_Q         0x141000
#define AS_W         0x141100
#define AS_E         0x141200
#define AS_R         0x141300
#define AS_T         0x141400
#define AS_Y         0x141500
#define AS_U         0x141600
#define AS_I         0x141700
#define AS_O         0x141800
#define AS_P         0x141900
#define AS_LBRACE    0x141A00
#define AS_RBRACE    0x141B00
#define AS_LINE      0x142B00
#define AS_A         0x141E00
#define AS_S         0x141F00
#define AS_D         0x142000
#define AS_F         0x142100
#define AS_G         0x142200
#define AS_H         0x142300
#define AS_J         0x142400
#define AS_K         0x142500
#define AS_L         0x142600
#define AS_COLON     0x142700
#define AS_QUOTE     0x142800
#define AS_ENTER     0x141C00
#define AS_Z         0x142C00
#define AS_X         0x142D00
#define AS_C         0x142E00
#define AS_V         0x142F00
#define AS_B         0x143000
#define AS_N         0x143100
#define AS_M         0x143200
#define AS_COMMA     0x143300
#define AS_PERIOD    0x143400
#define AS_QMARK     0x143500
#define AS_SPACE     0x143920
#define AS_INS       0x14A200
#define AS_HOME      0x149700
#define AS_PGUP      0x149900
#define AS_DEL       0x14A300
#define AS_END       0x149F00
#define AS_PGDN      0x14A100
#define AS_UP        0x149800
#define AS_LEFT      0x149B00
#define AS_DOWN      0x14A000
#define AS_RIGHT     0x149D00
#define AS_KP_DIV    0x14A400
#define AS_KP_MULT   0x143700
#define AS_KP_MINUS  0x144A00
#define AS_KP_PLUS   0x144E00
#define AS_KP_ENTER  0x14A600

// CTRL_SHIFT key codes
#define CS_ESC       0x0C011B
#define CS_F1        0x0C5E00
#define CS_F2        0x0C5F00
#define CS_F3        0x0C6000
#define CS_F4        0x0C6100
#define CS_F5        0x0C6200
#define CS_F6        0x0C6300
#define CS_F7        0x0C6400
#define CS_F8        0x0C6500
#define CS_F9        0x0C6600
#define CS_F10       0x0C6700
#define CS_F11       0x0C8900
#define CS_F12       0x0C8A00
#define CS_2         0x0C0300
#define CS_6         0x0C071E
#define CS_DASH      0x0C0C1F
#define CS_BACKSP    0x0C0E7F
#define CS_TAB       0x0C9400
#define CS_Q         0x0C1011
#define CS_W         0x0C1117
#define CS_E         0x0C1205
#define CS_R         0x0C1312
#define CS_T         0x0C1414
#define CS_Y         0x0C1519
#define CS_U         0x0C1615
#define CS_I         0x0C1709
#define CS_O         0x0C180F
#define CS_P         0x0C1910
#define CS_LBRACE    0x0C1A1B
#define CS_RBRACE    0x0C1B1D
#define CS_LINE      0x0C2B1C
#define CS_A         0x0C1E01
#define CS_S         0x0C1F13
#define CS_D         0x0C2004
#define CS_F         0x0C2106
#define CS_G         0x0C2207
#define CS_H         0x0C2308
#define CS_J         0x0C240A
#define CS_K         0x0C250B
#define CS_L         0x0C260C
#define CS_ENTER     0x0C1C0A
#define CS_Z         0x0C2C1A
#define CS_X         0x0C2D18
#define CS_C         0x0C2E03
#define CS_V         0x0C2F16
#define CS_B         0x0C3002
#define CS_N         0x0C310E
#define CS_M         0x0C320D
#define CS_SPACE     0x0C3920
#define CS_INS       0x0C92E0
#define CS_HOME      0x0C77E0
#define CS_PGUP      0x0C84E0
#define CS_DEL       0x0C93E0
#define CS_END       0x0C75E0
#define CS_PGDN      0x0C76E0
#define CS_UP        0x0C8DE0
#define CS_LEFT      0x0C73E0
#define CS_DOWN      0x0C91E0
#define CS_RIGHT     0x0C74E0
#define CS_KP_DIV    0x0C9500
#define CS_KP_MULT   0x0C9600
#define CS_KP_MINUS  0x0C8E00
#define CS_KP_PLUS   0x0C9000
#define CS_KP_ENTER  0x0CE00A
#define CS_KP_HOME   0x0C7700
#define CS_KP_UP     0x0C8D00
#define CS_KP_PGUP   0x0C8400
#define CS_KP_LEFT   0x0C7300
#define CS_KP_CENTER 0x0C8F00
#define CS_KP_RIGHT  0x0C7400
#define CS_KP_END    0x0C7500
#define CS_KP_DOWN   0x0C9100
#define CS_KP_PGDN   0x0C7600
#define CS_KP_INS    0x0C9200
#define CS_KP_DEL    0x0C9300

// CTRL_ALT_SHIFT key codes
#define CAS_ESC      0x1C0100
#define CAS_F1       0x1C6800
#define CAS_F2       0x1C6900
#define CAS_F3       0x1C6A00
#define CAS_F4       0x1C6B00
#define CAS_F5       0x1C6C00
#define CAS_F6       0x1C6D00
#define CAS_F7       0x1C6E00
#define CAS_F8       0x1C6F00
#define CAS_F9       0x1C7000
#define CAS_F10      0x1C7100
#define CAS_F11      0x1C8B00
#define CAS_F12      0x1C8C00
#define CAS_TILDE    0x1C2900
#define CAS_1        0x1C7800
#define CAS_2        0x1C7900
#define CAS_3        0x1C7A00
#define CAS_4        0x1C7B00
#define CAS_5        0x1C7C00
#define CAS_6        0x1C7D00
#define CAS_7        0x1C7E00
#define CAS_8        0x1C7F00
#define CAS_9        0x1C8000
#define CAS_0        0x1C8100
#define CAS_DASH     0x1C8200
#define CAS_PLUS     0x1C8300
#define CAS_BACKSP   0x1C0E00
#define CAS_TAB      0x1CA500
#define CAS_Q        0x1C1000
#define CAS_W        0x1C1100
#define CAS_E        0x1C1200
#define CAS_R        0x1C1300
#define CAS_T        0x1C1400
#define CAS_Y        0x1C1500
#define CAS_U        0x1C1600
#define CAS_I        0x1C1700
#define CAS_O        0x1C1800
#define CAS_P        0x1C1900
#define CAS_LBRACE   0x1C1A00
#define CAS_RBRACE   0x1C1B00
#define CAS_LINE     0x1C2B00
#define CAS_A        0x1C1E00
#define CAS_S        0x1C1F00
#define CAS_D        0x1C2000
#define CAS_F        0x1C2100
#define CAS_G        0x1C2200
#define CAS_H        0x1C2300
#define CAS_J        0x1C2400
#define CAS_K        0x1C2500
#define CAS_L        0x1C2600
#define CAS_COLON    0x1C2700
#define CAS_QUOTE    0x1C2800
#define CAS_ENTER    0x1C1C00
#define CAS_Z        0x1C2C00
#define CAS_X        0x1C2D00
#define CAS_C        0x1C2E00
#define CAS_V        0x1C2F00
#define CAS_B        0x1C3000
#define CAS_N        0x1C3100
#define CAS_M        0x1C3200
#define CAS_COMMA    0x1C3300
#define CAS_PERIOD   0x1C3400
#define CAS_QMARK    0x1C3500
#define CAS_SPACE    0x1C3920
#define CAS_INS      0x1CA200
#define CAS_HOME     0x1C9700
#define CAS_PGUP     0x1C9900
#define CAS_END      0x1C9F00
#define CAS_PGDN     0x1CA100
#define CAS_UP       0x1C9800
#define CAS_LEFT     0x1C9B00
#define CAS_DOWN     0x1CA000
#define CAS_RIGHT    0x1C9D00
#define CAS_KP_DIV   0x1CA400
#define CAS_KP_MULT  0x1C3700
#define CAS_KP_MINUS 0x1C4A00
#define CAS_KP_PLUS  0x1C4E00
#define CAS_KP_ENTER 0x1CA600
                         
// Combined SHIFT constants
#define SHIFT          0x0400
#define CTRL           0x0800
#define ALT            0x1000

// SHIFT status flags
#define RT_SHIFT_DOWN  0x0001
#define LT_SHIFT_DOWN  0x0002
#define SHIFT_DOWN     0x0003
#define CTRL_DOWN      0x0004
#define ALT_DOWN       0x0008
#define SCRL_LOCKED    0x0010
#define NUM_LOCKED     0x0020
#define CAPS_LOCKED    0x0040
#define INSERT_ON      0x0080
#define LT_CTRL_DOWN   0x0100
#define LT_ALT_DOWN    0x0200
#define RT_CTRL_DOWN   0x0400
#define RT_ALT_DOWN    0x0800
#define SCRL_DOWN      0x1000
#define NUM_DOWN       0x2000
#define CAPS_DOWN      0x4000
#define SYS_DOWN       0x8000
#endif  // __OP_KEY_H
