//--------------------------------------------------
// Mouse class
//--------------------------------------------------
// $Revision:   1.1  $
// $Date:   05 Apr 1993 18:18:56  $
//--------------------------------------------------
#ifndef __OP_MOUSE_H
#define __OP_MOUSE_H
#ifndef __DOS_H
#include <dos.h>
#endif
#ifndef __OP_GRAPH_H
#include "op_graph.h"
#endif

#define LEFT_BUTTON   0
#define RIGHT_BUTTON  1
#define MID_BUTTON    2
#define STD_DRAW      0
#define ASM_DRAW      1
#define BGI_DRAW      2
#define M_INT        51

//---------------------------------------------------
// Mouse control structures
//---------------------------------------------------
struct g_cursor
{
 unsigned *image;
 unsigned xkey;
 unsigned ykey;
};

//---------------------------------------------------
// Mouse Class
//---------------------------------------------------
class Mouse
{
 private:
  union REGS inreg;
  union REGS outreg;
  Graphics *gr;
  int showing;
  void graphic_cursor(int x, int y,
                      unsigned mask_Seg, unsigned mask_Ofs);

 public:
  int draw_method;
  char *bmp_cursor;
  int x_ofs, y_ofs;
  int hot_x, hot_y;
  char *bit_ptr;
  char *saved_screen;
  int active;
  int buttons;
  int button_status;
  int button_count;
  int x_pos, y_pos;
  int old_x, old_y;
  int x_count;
  int y_count;
  int paging;
  int host_port;

  // Setup functions
  Mouse(Graphics *graphics);
  void activate(void);
  void prepare_cursor(char *bmap);
  void set_hotspot(int x, int y);
  void set_method(int method) { draw_method = method; }
  void text_cursor(int type, unsigned start, unsigned stop);
  void move_ratio(int xsize, int ysize);
  void range(int xmin, int ymin, int xmax, int ymax);
  void speed(int speed);
  void motion(void);
  void clip(int left, int top, int right, int bottom);
  void install_handler(void far *fn_name);

  // Query functions
  void read(void);
  int pressed(int button);
  int released(int button);
  int get_crt_pageno(void);

  // Alteration functions
  void moveto(int x, int y);
  void set_crt_pageno(int page);
  void set_port_no(int p_no) { host_port = p_no; }
  void show(void);
  void hide(void);
  void disable_text_cursor(void);
  void gr_cursor(g_cursor& curs, int hot_x = -1, int hot_y = -1);
  void bit_cursor(char *b_cursor) { bmp_cursor = b_cursor; }
  void asm_draw(void);
  void asm_update(void);
  void asm_erase(void);
  void bgi_draw(void);
  void bgi_erase(void);
  void bgi_update(void);
};

extern g_cursor ARROW;
extern g_cursor CHECK;
extern g_cursor CROSS;
extern g_cursor GLOVE;
extern g_cursor IBEAM;
#endif  // __OP_MOUSE_H
 
